SystemOrganization addCategory: #'Pier-TagCloud'!

!PRStructure class methodsFor: '*pier-tagcloud-descriptions' stamp: 'lr 9/13/2007 10:21'!
descriptionMenuTagCloud: aDescription
	^ aDescription beTagCloud! !

!PRStructure class methodsFor: '*pier-tagcloud-descriptions' stamp: 'lr 9/13/2007 10:21'!
descriptionNameTagCloud: aDescription
	^ aDescription beTagCloud! !

!PRStructure class methodsFor: '*pier-tagcloud-descriptions' stamp: 'lr 9/13/2007 10:20'!
descriptionTagsTagCloud: aDescription
	^ aDescription beTagCloud! !

!PRStructure class methodsFor: '*pier-tagcloud-descriptions' stamp: 'lr 9/13/2007 10:20'!
descriptionTitleTagCloud: aDescription
	^ aDescription beTagCloud! !

!PRStructure methodsFor: '*pier-tagcloud' stamp: 'lr 9/13/2007 10:09'!
tagsUsing: aCollection
	"Answer a list of tags using the given aCollection of descriptions."
	
	^ aCollection
		inject: OrderedCollection new
		into: [ :collection :each |
			collection addAll: ((each toString: (self readUsing: each))
				findTokens: ' ').
			collection ]! !

PRWidget subclass: #PRTagCloudWidget
	instanceVariableNames: 'tags tagsByCount tagsByName focus'
	classVariableNames: 'StopWords'
	poolDictionaries: ''
	category: 'Pier-TagCloud'!

PRTagCloudWidget subclass: #PRExponentialTagCloudWidget
	instanceVariableNames: 'mean deviation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-TagCloud'!

!PRExponentialTagCloudWidget class methodsFor: 'accessing' stamp: 'cdlm 11/11/2007 05:28'!
label
	^ 'Tag Cloud (exponential)'! !

!PRExponentialTagCloudWidget methodsFor: 'private' stamp: 'cdlm 11/18/2007 19:47'!
computeSizeParameters
	| numberOfTags |
	numberOfTags := self tagsByCount size max: 1. "avoid dividing by zero"
	mean := 0.0.
	deviation := 0.0.
	self tagsByCount do:
		[ :each | mean := mean + each value size ].
	mean := mean / numberOfTags.
	self tagsByCount do:
		[ :each | deviation := deviation + (mean - each value size) abs ].
	deviation := deviation / numberOfTags.
	deviation = 0.0 ifTrue: [ deviation := 1.0 ]! !

!PRExponentialTagCloudWidget methodsFor: 'rendering' stamp: 'cdlm 11/11/2007 05:04'!
renderContentOn: html
	self computeSizeParameters.
	super renderContentOn: html! !

!PRExponentialTagCloudWidget methodsFor: 'rendering' stamp: 'cdlm 11/11/2007 05:31'!
renderTag: anAssociation on: html
	| temp percent |
	temp := ((anAssociation value size - mean) / deviation) exp.
	percent := ((temp - 1) / (temp + 1) * (1.8 ln)) exp * 100.0.
	html span class: 'tag'; with: [
		html anchor
			callback: [ self focus: anAssociation ];
			style: 'font-size: ' , percent truncated asString , '%';
			with: anAssociation key.
		(self read: #descriptionShowFrequencies) ifTrue: [
			html span class: 'freq'; with: [
				html space; text: '('; render: anAssociation value size; text: ')' ] ] ]! !

!PRTagCloudWidget class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionDataDescriptions
	| optionsAndLabels |
	optionsAndLabels := Dictionary new.
	PRStructure withAllSubclassesDo: 
		[ :class | 
		class description do: 
			[ :each | 
			each isTagCloud ifTrue: 
				[ optionsAndLabels 
					at: each
					put: each label ] ] ].
	^ MAMultipleOptionDescription new
		optionsAndLabels: optionsAndLabels associations;
		reference: MADescription description;
		accessor: #dataDescriptions;
		default: optionsAndLabels keys;
		priority: 220;
		label: 'Data';
		beDistinct;
		yourself! !

!PRTagCloudWidget class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionIgnoredTags
	^ MAMemoDescription new
		accessor: #ignoredTags;
		label: 'Ignored tags';
		priority: 220;
		yourself! !

!PRTagCloudWidget class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionMaxItems
	^ MANumberDescription new
		accessor: #maxItems;
		label: 'Max Items';
		priority: 220;
		default: 50;
		beInteger;
		min: 1;
		yourself! !

!PRTagCloudWidget class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionShowFrequencies
	^ MABooleanDescription new
		accessor: #showFrequencies;
		checkboxLabel: 'Frequencies';
		default: false;
		label: 'Show';
		priority: 230;
		yourself! !

!PRTagCloudWidget class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionStructure
	^ PRStructureDescription new
		accessor: #startAt;
		label: 'Start At';
		priority: 200;
		yourself! !

!PRTagCloudWidget class methodsFor: 'initialization' stamp: 'lr 9/14/2007 13:39'!
initialize
	StopWords := Set withAll: #('--' '---' 'about' 'an' 'and' 'are' 'as' 'at' 'be' 'by' 'com' 'de' 'en' 'for' 'from' 'he' 'her' 'hers' 'herself' 'him' 'himself' 'how' 'in' 'is' 'it' 'its' 'itself' 'la' 'me' 'mine' 'my' 'myself' 'of' 'on' 'oneself' 'or' 'our' 'ours' 'ourselves' 'she' 'that' 'the' 'their' 'theirs' 'them' 'themselves' 'these' 'they' 'this' 'those' 'to' 'und' 'us' 'was' 'we' 'what' 'when' 'where' 'who' 'will' 'with' 'www' 'you' 'your' 'yours' 'yourself' 'yourselves')! !

!PRTagCloudWidget class methodsFor: 'testing' stamp: 'lr 9/12/2007 22:55'!
isAbstract
	^ false! !

!PRTagCloudWidget class methodsFor: 'accessing' stamp: 'lr 9/12/2007 22:54'!
label
	^ 'Tag Cloud'! !

!PRTagCloudWidget methodsFor: 'accessing' stamp: 'lr 9/12/2007 22:32'!
enumerator
	"Answer an enumerator to visit the necessary structures to collect the tags."
	
	^ (self context enumeratorFor: self structure) with all! !

!PRTagCloudWidget methodsFor: 'actions' stamp: 'lr 9/14/2007 17:56'!
focus: anAssocation
	focus := anAssocation! !

!PRTagCloudWidget methodsFor: 'accessing' stamp: 'tg 11/17/2007 00:40'!
ignoredTags
	^(self read: #descriptionIgnoredTags) ifNil: ['']! !

!PRTagCloudWidget methodsFor: 'rendering' stamp: 'lr 9/14/2007 19:01'!
renderCloudOn: html
	self tagsByName 
		do: [ :assoc | self renderTag: assoc on: html ]
		separatedBy: [ html text: ' ' ]! !

!PRTagCloudWidget methodsFor: 'rendering' stamp: 'lr 9/14/2007 17:57'!
renderContentOn: html
	focus isNil
		ifTrue: [ self renderCloudOn: html ]
		ifFalse: [ self renderFocusOn: html ]! !

!PRTagCloudWidget methodsFor: 'rendering' stamp: 'lr 9/17/2007 08:40'!
renderFocusOn: html
	| structures related |
	structures := focus value asSet
		asSortedCollection: [ :a :b | a title < b title ].
	html heading level: 1; with: [ 
		html anchor
			callback: [ focus := nil ];
			with: focus key ].
	html definitionList: [
		structures do: [ :each |
			html definitionTerm: [
				html anchor
					callback: [ 
						self context: (self context structure: each).
						focus := nil ];
					with: each title ].
			html definitionData: [
				related := self tagsByCount
					select: [ :assoc | assoc key ~= focus key and: [ assoc value includes: each ] ].
				((related last: (5 min: related size))
					asSortedCollection: [ :a :b | a key < b key ])
					do: [ :assoc | self renderTag: assoc on: html ]
					separatedBy: [ html text: ' ' ] ] ] ]! !

!PRTagCloudWidget methodsFor: 'rendering' stamp: 'lr 8/29/2008 11:59'!
renderTag: anAssocation on: html
	| min max diff percent |
	min := self tagsByCount first value size.
	max := self tagsByCount last value size.
	(diff := max - min) isZero
		ifTrue: [ diff := 1 ].
	percent := (((anAssocation value size - min) * 100.0 / diff) + 100)
		truncated.
	html span class: 'tag'; with: [
		html anchor
			callback: [ self focus: anAssocation ];
			style: 'font-size: ' , percent asString , '%';
			with: anAssocation key.
		(self read: #descriptionShowFrequencies) ifTrue: [
			html span
				class: 'freq';
				with: [ html space; text: '('; render: anAssocation value size; text: ')' ] ] ]! !

!PRTagCloudWidget methodsFor: 'actions' stamp: 'lr 9/14/2007 17:55'!
reset
	tags := tagsByName := tagsByCount := focus := nil! !

!PRTagCloudWidget methodsFor: 'accessing' stamp: 'lr 9/12/2007 22:23'!
structure
	"Answer the structure where the search for the tag cloud should start from."
	
	^ (self read: #descriptionStructure) ifNil: [ self context structure ]! !

!PRTagCloudWidget methodsFor: 'accessing' stamp: 'tg 11/19/2007 17:49'!
tags
	"Answer a dictionary mapping tags to structures."
	
	tags isNil
		ifFalse: [ ^ tags ].
	tags := Dictionary new.
	self enumerator do: [ :struct |
		(self read: #descriptionDataDescriptions) do: [ :description |
			(self tokenize: struct using: description) do: [ :tag |
				(tags at: tag ifAbsentPut: [ OrderedCollection new ])
					add: struct ] ] ].
	^ tags! !

!PRTagCloudWidget methodsFor: 'accessing' stamp: 'lr 9/14/2007 17:53'!
tagsByCount
	"Answer a dictionary mapping tags to structures. The tags are sorted by count and cut to a certain threshold."
	
	tagsByCount isNil
		ifFalse: [ ^ tagsByCount ].
	tagsByCount := SortedCollection
		sortBlock: [ :a :b | a value size < b value size ].
	self tags associationsDo: [ :assoc | tagsByCount add: assoc ].
	(self read: #descriptionMaxItems) isNil
		ifFalse: [ tagsByCount := tagsByCount last: (tagsByCount size min: (self read: #descriptionMaxItems)) ].
	^ tagsByCount! !

!PRTagCloudWidget methodsFor: 'accessing' stamp: 'lr 9/14/2007 18:05'!
tagsByName
	"Answer the tags alphabetically sorted by name."
	
	tagsByName isNil
		ifFalse: [ ^ tagsByName ].
	^ tagsByName := self tagsByCount 
		asSortedCollection: [ :a :b | a key < b key ]! !

!PRTagCloudWidget methodsFor: 'private' stamp: 'tg 11/17/2007 00:32'!
tokenize: anObject using: aDescription
	| string tokens ignoredTokens |
	(aDescription accessor canRead: anObject)
		ifFalse: [ ^ #() ].
	string := aDescription toString: (anObject readUsing: aDescription).
	string := string collect: [ :each | 
		each isLetter
			ifTrue: [ each asLowercase ]
			ifFalse: [ 
				each = $-
					ifTrue: [ each ]
					ifFalse: [ $  ] ] ].
	tokens := string findTokens: ' '.
	ignoredTokens := self ignoredTags findTokens: ' '.
	^ tokens reject: [ :each | each size = 1 or: [ (StopWords includes: each) or: [ignoredTokens includes: each] ] ]
! !

!GRPackage class methodsFor: '*pier-tagcloud' stamp: 'lr 2/18/2010 11:55'!
pierTagCloud
	^ self new
		name: 'Pier-TagCloud';
		description: 'A tag cloud for Pier.';
		addDependency: 'Pier-Model';
		addDependency: 'Pier-Seaside';
		url: #pierUrl;
		yourself! !

!PRCase class methodsFor: '*pier-tagcloud-descriptions' stamp: 'NickAger 2/22/2010 00:24'!
descriptionPlainText
	^ MAStringDescription new
		accessor: #plainText;
		label: 'Plain Text';
		beSearchable;
		beTagCloud;
		readonly;
		priority: 100;
		yourself! !

!MADescription methodsFor: '*pier-tagcloud' stamp: 'lr 9/13/2007 10:22'!
beTagCloud
	self propertyAt: #tagCloud put: true! !

!MADescription methodsFor: '*pier-tagcloud' stamp: 'lr 9/13/2007 10:22'!
isTagCloud
	^ self propertyAt: #tagCloud ifAbsent: [ false ]! !
PRTagCloudWidget initialize!
